function k = kernel(knl, kpar, X1, X2)
%KERNEL Calculates a kernel matrix.
%   K = KERNEL(KNL, KPAR, X1, X2) calculates the nxN kernel matrix given
%   two matrix X1[n,d], X2[N,d] with kernel type specified by 'knl':
%       'lin'   - linear kernel, 'kpar' is not considered
%       'pol'   - polinomial kernel, where 'kpar' is the polinomial degree
%       'gauss' - gaussian kernel, where 'kpar' is the gaussian sigma
%
%   Example:
%       X1 = randn(n, d);
%       X2 = randn(N, d);
%       K = kernel('lin', [], X1, X2);
%       K = kernel('gauss', 2.0, X1, X2);
%
% See also LEARN

N = size(X1, 1);
n = size(X2, 1);

switch lower(knl)
   
    case 'lin'
        k = X1 * X2';
        
    case 'pol'
        deg = kpar;
        if (int64(deg) ~= deg)
            error('polynomial kernel degree should be an integer')
        end
        k = (X1 * X2') .^ deg;
        
    case 'gauss'
        sigma = kpar;
        if (sigma <= 0)
            error('gaussian kernel sigma should be greater than zero')
        end
        [a b] = meshgrid(1:n, 1:N);
        a = a(:);
        b = b(:);
        var = (2 * (sigma*sigma));
        out = exp( -sum((X1(b,:)-X2(a,:)).^2, 2) / var);
        k = reshape(out,N,n);
        
    otherwise
        error('Unknown kernel')
end
